﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using Lassalle.Flow;
using Lassalle.Flow.Layout.Hierarchic;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 이미지 딕셔너리
        /// </summary>
        private Dictionary<NodeType, Image> imageDictionary;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.imageDictionary = new Dictionary<NodeType, Image>();

            this.imageDictionary.Add(NodeType.DATA   , Properties.Resources.data   );
            this.imageDictionary.Add(NodeType.FILTER , Properties.Resources.filter );
            this.imageDictionary.Add(NodeType.GROUPBY, Properties.Resources.groupBy);
            this.imageDictionary.Add(NodeType.MERGE  , Properties.Resources.merge  );
            this.imageDictionary.Add(NodeType.QUERY  , Properties.Resources.query  );
            this.imageDictionary.Add(NodeType.SORT   , Properties.Resources.sort   );

            this.addFlow.Images.Add(new FlowImage(this.imageDictionary[NodeType.DATA   ]));
            this.addFlow.Images.Add(new FlowImage(this.imageDictionary[NodeType.FILTER ]));
            this.addFlow.Images.Add(new FlowImage(this.imageDictionary[NodeType.GROUPBY]));
            this.addFlow.Images.Add(new FlowImage(this.imageDictionary[NodeType.MERGE  ]));
            this.addFlow.Images.Add(new FlowImage(this.imageDictionary[NodeType.QUERY  ]));
            this.addFlow.Images.Add(new FlowImage(this.imageDictionary[NodeType.SORT   ]));

            this.treeView.Nodes.Add(new TreeNode { Text = NodeType.DATA.ToString()   , Tag = NodeType.DATA   });
            this.treeView.Nodes.Add(new TreeNode { Text = NodeType.FILTER.ToString() , Tag = NodeType.FILTER });
            this.treeView.Nodes.Add(new TreeNode { Text = NodeType.GROUPBY.ToString(), Tag = NodeType.GROUPBY});
            this.treeView.Nodes.Add(new TreeNode { Text = NodeType.MERGE.ToString()  , Tag = NodeType.MERGE  });
            this.treeView.Nodes.Add(new TreeNode { Text = NodeType.QUERY.ToString()  , Tag = NodeType.QUERY  });
            this.treeView.Nodes.Add(new TreeNode { Text = NodeType.SORT.ToString()   , Tag = NodeType.SORT   });

            this.addFlow.AutoScroll              = true;
            this.addFlow.AllowDrop               = true;
            this.addFlow.CanDragScroll           = true;
            this.addFlow.CanDrawNode             = false;
            this.addFlow.CanSizeNode             = false;
            this.addFlow.CanLabelEdit            = false;
            this.addFlow.CanReflexLink           = false;
            this.addFlow.PageUnit                = GraphicsUnit.Pixel;
            this.addFlow.DefNodeProp.FillColor   = Color.White;
            this.addFlow.DefNodeProp.Shape.Style = Lassalle.Flow.ShapeStyle.Data; 

            this.treeView.ItemDrag     += treeView_ItemDrag;
            this.layoutButton.Click    += layoutButton_Click;
            this.addFlow.DragEnter     += addFlow_DragEnter;
            this.addFlow.DragDrop      += addFlow_DragDrop;
            this.addFlow.BeforeAddLink += addFlow_BeforeAddLink;
            this.addFlow.AfterAddLink  += addFlow_AfterAddLink;
            this.addFlow.KeyDown       += addFlow_KeyDown;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 트리 뷰 항목 DRAG 처리하기 - treeView_ItemDrag(sender, e)

        /// <summary>
        /// 트리 뷰 항목 DRAG 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void treeView_ItemDrag(object sender, ItemDragEventArgs e)
        {
            this.addFlow.DoDragDrop(e.Item, DragDropEffects.Move);
        }

        #endregion
        #region Layout 버튼 클릭시 처리하기 - layoutButton_Click(sender, e)

        /// <summary>
        /// Layout 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void layoutButton_Click(object sender, EventArgs e)
        {
            HFlow flow = new HFlow();

            flow.LayerDistance  = 50;
            flow.VertexDistance = 50;
            flow.Orientation    = Lassalle.Flow.Layout.Hierarchic.Orientation.North;
            flow.LayerWidth     = 0;

            flow.Layout(this.addFlow);


            for(int i = 0; i < this.addFlow.Nodes.Count; i++)
            {
                Node node = this.addFlow.Nodes[i];
            
                if(node.Parent != null)
                {
                    Node parentNode = node.Parent as Node;
            
                  //float textX = parentNode.Rect.Right + 5f;
                  //float textY = parentNode.Rect.Bottom - node.Font.Height - 5f;

                    float textX = parentNode.Rect.Left + (parentNode.Rect.Width - node.Rect.Width) / 2f;
                    float textY = parentNode.Rect.Bottom + 5f;
            
            
                    node.Rect = new RectangleF(textX, textY, node.Rect.Width, node.Rect.Height);
                }
            }
        }

        #endregion

        #region AddFlow DRAG ENTER 처리하기 - addFlow_DragEnter(sender, e)

        /// <summary>
        /// AddFlow DRAG ENTER 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addFlow_DragEnter(object sender, DragEventArgs e)
        {
            TreeNode node = e.Data.GetData("System.Windows.Forms.TreeNode", true) as TreeNode;

            if(node != null)
            {
                e.Effect = DragDropEffects.Move;
            }
        }

        #endregion
        #region AddFlow DRAG & DROP 처리하기 - addFlow_DragDrop(sender, e)

        /// <summary>
        /// AddFlow DRAG & DROP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addFlow_DragDrop(object sender, DragEventArgs e)
        {
            TreeNode treeNode = e.Data.GetData("System.Windows.Forms.TreeNode", true) as TreeNode;

            if(treeNode != null)
            {
                NodeType nodeType = (NodeType)treeNode.Tag;

                Point clientPoint  = this.addFlow.PointToClient(new Point(e.X, e.Y));
                Point controlPoint = this.addFlow.PointToAddFlow(clientPoint);

                AddNode(this.addFlow, controlPoint.X, controlPoint.Y, 42, 42, nodeType);
            }
        }

        #endregion
        #region AddFlow KEY DOWN 처리하기 - addFlow_KeyDown(sender, e)

        /// <summary>
        /// AddFlow KEY DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addFlow_KeyDown(object sender, KeyEventArgs e)
        {
            if(e.KeyCode == Keys.Delete)
            {
                if(this.addFlow.SelectedItems == null || this.addFlow.SelectedItems.Count == 0)
                {
                    return;
                }

                List<Node> nodeList = new List<Node>();
                List<Link> linkList = new List<Link>();

                for(int i = 0; i < this.addFlow.SelectedItems.Count; i++)
                {
                    object item = this.addFlow.SelectedItems[i];

                    if(item is Node)
                    {
                        nodeList.Add(item as Node);
                    }

                    if(item is Link)
                    {
                        linkList.Add(item as Link);
                    }
                }

                foreach(Node node in nodeList)
                {
                    foreach(Link link in node.Links)
                    {
                        node.Links.Remove(link);
                    }

                    this.addFlow.Items.Remove(node);
                }

                foreach(Link link in linkList)
                {
                    if(nodeList.Contains(link.Org) || nodeList.Contains(link.Dst))
                    {
                        continue;
                    }

                    BaseNode startBaseNode = link.Org as BaseNode;
                    BaseNode endBaseNode   = link.Dst as BaseNode;

                    startBaseNode.Links.Remove(link);

                    if(startBaseNode != null)
                    {
                        startBaseNode.Validate();
                    }

                    if(endBaseNode != null)
                    {
                        endBaseNode.Validate();
                    }
                }

                this.addFlow.SelectedItems.Clear();

                this.addFlow.Refresh();
            }
        }

        #endregion
        #region AddFlow 링크 추가 전 처리하기 - addFlow_BeforeAddLink(sender, e)

        /// <summary>
        /// AddFlow 링크 추가 전 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addFlow_BeforeAddLink(object sender, BeforeAddLinkEventArgs e)
        {
            Node startNode = e.Org;
            Node endNode   = e.Dst;

            if(endNode is QueryNode)
            {
                e.Cancel.Cancel = true;
            }
            else if(endNode is DataNode)
            {
                e.Cancel.Cancel = true;
            }
            else if(endNode is SortNode)
            {
                if(endNode.InLinks.Count > 0)
                {
                    e.Cancel.Cancel = true;
                }
            }
            else if(endNode is FilterNode)
            {
                if(endNode.InLinks.Count > 0)
                {
                    e.Cancel.Cancel = true;
                }
            }
            else if(endNode is MergeNode)
            {
                if(endNode.InLinks.Count > 1)
                {
                    e.Cancel.Cancel = true;
                }
            }
            else if(endNode is GroupByNode)
            {
                if(endNode.InLinks.Count > 0)
                {
                    e.Cancel.Cancel = true;
                }
            }
        }

        #endregion
        #region AddFlow 링크 추가 후 처리하기 - addFlow_AfterAddLink(sender, e)

        /// <summary>
        /// AddFlow 링크 추가 후 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addFlow_AfterAddLink(object sender, AfterAddLinkEventArgs e)
        {
            (e.Link.Org as BaseNode).Validate();
            (e.Link.Dst as BaseNode).Validate();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 노드 추가하기 - AddNode(addFlow, nodeX, nodeY, nodeWidth, nodeHeight, nodeType)

        /// <summary>
        /// 노드 추가하기
        /// </summary>
        /// <param name="addFlow">AddFlow</param>
        /// <param name="nodeX">노드 X 좌표</param>
        /// <param name="nodeY">노드 Y 좌표</param>
        /// <param name="nodeWidth">노드 너비</param>
        /// <param name="nodeHeight">노드 높이</param>
        /// <param name="nodeType">노드 타입</param>
        /// <returns>추가 노드</returns>
        private BaseNode AddNode(AddFlow addFlow, int nodeX, int nodeY, int nodeWidth, int nodeHeight, NodeType nodeType)
        {
            BaseNode baseNode = null;

            if     (nodeType == NodeType.DATA   ) baseNode = new DataNode();
            else if(nodeType == NodeType.FILTER ) baseNode = new FilterNode();
            else if(nodeType == NodeType.GROUPBY) baseNode = new GroupByNode();
            else if(nodeType == NodeType.MERGE  ) baseNode = new MergeNode();
            else if(nodeType == NodeType.QUERY  ) baseNode = new QueryNode();
            else if(nodeType == NodeType.SORT   ) baseNode = new SortNode();

            if(baseNode == null)
            {
                return null;
            }
            else
            {
                baseNode.Rect = new RectangleF(nodeX, nodeY, nodeWidth, nodeHeight);

                addFlow.Nodes.Add(baseNode);

                baseNode.Validate();

                AddLabelNode(addFlow, baseNode);

                return baseNode;
            }
        }

        #endregion
        #region 레이블 노드 추가하기 - AddLabelNode(addFlow, baseNode)

        /// <summary>
        /// 레이블 노드 추가하기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="baseNode">베이스 노드</param>
        /// <returns>추가 레이블 노드</returns>
        private Node AddLabelNode(AddFlow addFlow, BaseNode baseNode)
        {
            using(Graphics graphics = addFlow.CreateGraphics())
            {
                SizeF textSizeF = graphics.MeasureString(baseNode.Text, baseNode.Font);

                float textX = baseNode.Rect.Left   + (baseNode.Rect.Width - textSizeF.Width) / 2f;
                float textY = baseNode.Rect.Bottom + 5f;
                    
                Node labelNode = new Node(textX, textY, textSizeF.Width, textSizeF.Height);
                    
                labelNode.Shape.Style     = ShapeStyle.Rectangle;
                labelNode.FillColor       = Color.Transparent;
                labelNode.DrawColor       = Color.Transparent;
                labelNode.TextColor       = Color.Gray;
                labelNode.Font            = baseNode.Font;
                labelNode.Text            = baseNode.Text;
                labelNode.Logical         = false;
                labelNode.Selectable      = false;
                labelNode.InLinkable      = false;
                labelNode.OutLinkable     = false;
                labelNode.AutoSize        = Lassalle.Flow.AutoSize.NodeToText;
                labelNode.AttachmentStyle = AttachmentStyle.Item;
                labelNode.Alignment       = Alignment.CenterBOTTOM;
            
                addFlow.Nodes.Add(labelNode);

                labelNode.Parent = baseNode;

                return labelNode;
            }
        }

        #endregion
    }
}