﻿using System;
using System.Drawing;

using Lassalle.Flow;

namespace TestProject
{
    /// <summary>
    /// 베이스 노드 클래스
    /// </summary>
    public class BaseNode : Node
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 노드 ID
        /// </summary>
        private string nodeID;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 노드 ID - NodeID

        /// <summary>
        /// 노드 ID
        /// </summary>
        public string NodeID
        {
            get
            {
                return this.nodeID;
            }
        }

        #endregion
        #region 입력 링크 상태 - InLinkStatus

        /// <summary>
        /// 입력 링크 상태
        /// </summary>
        public bool InLinkStatus { get; protected set; }

        #endregion
        #region 출력 링크 상태 - OutLinkStatus

        /// <summary>
        /// 출력 링크 상태
        /// </summary>
        public bool OutLinkStatus { get; protected set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - BaseNode()

        /// <summary>
        /// 생성자
        /// </summary>
        public BaseNode()
        {
            this.nodeID = Guid.NewGuid().ToString().Replace("-", string.Empty).ToUpper();

            DrawWidth     = 2;
            Shape.Style   = ShapeStyle.RoundRect;
            FillColor     = Color.White;
            ImagePosition = ImagePosition.CenterMiddle;
            TextColor     = Color.Transparent;
            Font          = new Font("나눔고딕코딩", 12f, FontStyle.Bold);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 무결성 조사하기 - Validate()

        /// <summary>
        /// 무결성 조사하기
        /// </summary>
        /// <returns>무결성 조사 결과</returns>
        public virtual bool Validate()
        {
            InLinkStatus  = true;
            OutLinkStatus = true;

            return InLinkStatus;
        }

        #endregion
    }
}